#pragma once

#include <algorithm>
#include <cmath>
#include <cstdint>

#include "index_scorer.hpp"
namespace pisa {

template <typename Wand>
struct bm25: public index_scorer<Wand> {
    static constexpr float b = 0.4;
    static constexpr float k1 = 0.9;

    using index_scorer<Wand>::index_scorer;

    static float doc_term_weight(uint64_t freq, float norm_len)
    {
        auto f = static_cast<float>(freq);
        return f / (f + k1 * (1.0F - b + b * norm_len));
    }

    // IDF (inverse document frequency)
    static float query_term_weight(uint64_t df, uint64_t num_docs)
    {
        auto fdf = static_cast<float>(df);
        float idf = std::log((float(num_docs) - fdf + 0.5F) / (fdf + 0.5F));
        static const float epsilon_score = 1.0E-6;
        return std::max(epsilon_score, idf) * (1.0F + k1);
    }

    term_scorer_t term_scorer(uint64_t term_id) const override
    {
        auto term_len = this->m_wdata.term_posting_count(term_id);
        auto term_weight = query_term_weight(term_len, this->m_wdata.num_docs());
        auto s = [&, term_weight](uint32_t doc, uint32_t freq) {
            return term_weight * doc_term_weight(freq, this->m_wdata.norm_len(doc));
        };
        return s;
    }
};
}  // namespace pisa
